@@ -16,6 +16,9 @@ def ApplyScriptDefaults(Strng,Source,Target):
     if Target == 'Telugu' or Target == 'Kannada':
         Options += ['NasalToAnusvara','MToAnusvara']
 
+    elif Target == 'Hebrew':
+        Options += ['HebrewVetVav', 'HebrewnonFinalShort']
+
     elif Target == 'Malayalam':
         Options += ['MalayalamAnusvaraNasal', 'MToAnusvara', 'MalayalamremoveHistorical']
 
@@ -23,7 +26,7 @@ def ApplyScriptDefaults(Strng,Source,Target):
         Options += ['TamilNaToNNa','AnusvaraToNasal', 'TamilpredictDentaNa']
 
     elif Target == 'Bengali':
-        Options += ['VaToBa','YaToYYa','KhandaTa']
+        Options += ['VaToBa','YaToYYa']
 
     elif Target == 'MeeteiMayek':
         Options += ['MeeteiMayekremoveHistorical']
@@ -32,7 +35,7 @@ def ApplyScriptDefaults(Strng,Source,Target):
         Options += ['LimburemoveHistorical']
 
     elif Target == 'Assamese':
-        Options += ['YaToYYa','KhandaTa']
+        Options += ['YaToYYa']
 
     elif Target == 'Oriya':
         Options += ['OriyaVa','YaToYYa']
@@ -41,7 +44,7 @@ def ApplyScriptDefaults(Strng,Source,Target):
         Options += ['YaToYYa']
 
     elif Target == 'Gurmukhi':
-        Options += ['GurmukhiTippiBindu','GurmukhiTippiGemination']
+        Options += ['GurmukhiCandrabindu', 'GurmukhiTippiBindu','GurmukhiTippiGemination']
 
     elif Target == 'Saurashtra':
         Options += ['SaurashtraHaru']
