@@ -14,12 +14,271 @@ import functools
 
 ### Consider Adding Options to ignore Nukta etc for Gujarati bengali by default
 
-##
+## Todo
+## Fix Font links and names in description
 
 def default(Strng):
 
     return Strng
 
+def kawitan(Strng):
+
+    return Strng
+
+def sundapura(Strng):
+
+    return Strng
+
+def NasalTilde(Strng):
+    Strng = re.sub('(m̐|ṃ|ṁ)', '\u0303', Strng)
+
+    return Strng
+
+def verticalKana(Strng):
+
+    return Strng
+
+def verticalSiddham(Strng):
+
+    return Strng
+
+def vtobJapanese(txt):
+
+    return txt
+
+def tokushuon(txt):
+   txt = txt.replace('si', 'suxi').replace('zi', 'zuxi')
+   txt = txt.replace('yi','ixi')
+   txt = txt.replace('fy', 'fux')
+   txt = txt.replace('nye', 'nixe')
+
+   txt = re.sub('(?<![sc])hu', 'hoxu', txt)
+   txt = re.sub('(?<![sc])hye', 'hixe', txt)
+
+   return txt
+
+def JapanesePostProcess(src, tgt, txt, nativize, postoptions):
+    from aksharamukha.ScriptMap.NonIndic import kana2roman
+    import pykakasi
+    from . import PostOptions, Convert
+
+    txt = Convert.convertScript(txt, src, "Telugu")
+
+    txt = txt.replace('ˆ', '')
+    txt = Convert.convertScript(txt.lower(), "ISO", "Inter")
+
+    txt = Convert.convertScript(txt, "Telugu", "RomanKana")
+
+    # Visarga
+    txt = re.sub('([aiueo])' + r'\1' + 'H', r'\1' + r'\1' + 'h' + r'\1', txt)
+
+    txt = re.sub('([aiueo])H', r'\1' + 'h' + r'\1', txt)
+
+    # nasalization
+    txt = txt.replace('Gk', 'nk').replace('Gg', 'ng').replace('Jc', 'nc').replace('Jj', 'nj').replace('mb', 'nb').replace('mp', 'np')
+
+    txt = txt.replace("nn", 'nnn').replace('c', 'ch').replace('chch', 'cch').replace('shsh', 'ssh').replace("mm", "nm")
+
+    txt = txt.replace(',', '、').replace('\uEA01', '。').replace('\uEA02', '。。')
+
+    txt = txt.replace('JJ', 'nnny')
+    txt = txt.replace('J', 'ny')
+
+    if 'vtobJapanese' in postoptions:
+        txt = txt.replace('v', 'b')
+
+    # fix tra, dra
+    txt = txt.replace('tr', 'tor').replace('dr', 'dor').replace('Dr', 'dor').replace('Tr', 'tor')
+
+    ## how to satya, sadya, sahya, asya, ask the person
+    txt = txt.replace('tya', 'tiya').replace('dya', 'diya').replace('sya', 'suya').replace('shya', 'shuya').replace('chya', 'chuya')#.replace('hya', 'hiya')
+    txt = txt.replace('di', 'dexi').replace('du', 'doxu')#.replace('dyu','dexyu')
+    txt = txt.replace('ti', 'texi').replace('tu', 'toxu')
+    #txt = txt.replace('kwi', 'kuxi').replace('kwe', 'kuxwe').replace('kwo', 'kuxwo').replace('kwa', 'kuxwa')
+    txt = txt.replace('mye', 'mixe').replace('pye', 'pixe').replace('bye', 'bixe')
+    txt = txt.replace('ye', 'ixe')
+    txt = txt.replace('vye', 'vuxixe').replace('vy', 'vuxy')
+    txt = txt.replace('she', 'shixe')
+
+    #print(txt)
+    if not nativize:
+        txt = re.sub('(r)(r\u309A)', 'rur\u309A', txt)
+        txt = re.sub('(r\u309A)(r\u309A)', 'rr' + '\u309A', txt)
+        txt = re.sub('(k\u309A)(k\u309A)', 'kk' + '\u309A', txt)
+        txt = re.sub('([rk])(\u309A)([aieou])', r'\1\3\2', txt)
+
+        txt = tokushuon(txt)
+    else:
+        #txt = txt.replace('v', 'w')
+        txt = txt.replace('r\u309A', 'r').replace('k\u309Ak' + '\u309A', 'ng').replace('k\u309A', 'ng')
+
+
+        txt = txt.replace('yi', 'i').replace('ye', 'e').replace('wu', 'u')
+        txt = txt.replace('wo', 'uxo')
+
+        # she
+        txt = txt.replace('she', 'shie')
+
+    #print(txt)
+
+    if tgt == 'Hiragana':
+        txt = kana2roman.to_hiragana(txt)
+
+        txt = re.sub('(k|g|ch|j|p|b|m|y|r|w|sh|s|h|z|f)' + '(' + r'\1' + ')', r'\1' + 'u', txt)
+
+        txt = re.sub('(d|t)' + '(' + r'\1' + ')', r'\1' + 'o', txt)
+
+        if not nativize:
+            txt = tokushuon(txt)
+
+        txt = kana2roman.to_hiragana(txt)
+
+        txt = re.sub('(k|g|ch|j|p|b|m|y|r|sh|s|h|z|f|v)', r'\1' + 'u', txt)
+        txt = re.sub('(d|t)', r'\1' + 'o', txt)
+
+        if not nativize:
+            txt = tokushuon(txt)
+
+        txt = kana2roman.to_hiragana(txt)
+
+        txt = txt.replace('う゛', 'ゔ')
+
+    if tgt == 'Katakana':
+        txt = txt.replace('aa', 'a-').replace('ii', 'i-').replace('ee', 'e-').replace('oo', 'o-').replace('uu','u-')
+        txt = txt.replace('a\u309Aa', 'a\u309A-').replace('i\u309Ai', 'i\u309A-').replace('e\u309Ae', 'e\u309A-').replace('o\u309Ao', 'o\u309A-').replace('u\u309Au','u\u309A-')
+
+        txt = kana2roman.to_katakana(txt)
+
+        txt = re.sub('(k|g|ch|j|p|b|m|y|r|sh|s|h|z|f|v)' + '(' + r'\1' + ')', r'\1' + 'u', txt)
+        txt = re.sub('(d|t)' + '(' + r'\1' + ')', r'\1' + 'o', txt)
+
+        if not nativize:
+            txt = tokushuon(txt)
+
+        txt = kana2roman.to_katakana(txt)
+        txt = re.sub('(k|g|ch|j|p|b|m|y|r|sh|s|h|z|f|v)', r'\1' + 'u', txt)
+        txt = re.sub('(d|t)', r'\1' + 'o', txt)
+
+        if not nativize:
+            txt = tokushuon(txt)
+
+        txt = kana2roman.to_katakana(txt)
+
+    txt = Convert.convertScript(txt, "Inter", "ISO")
+
+    return txt
+
+def urduRemoveInherent(Strng):
+    Strng = re.sub('\Ba', '', Strng)
+
+    return Strng
+
+def HebrewVetVav(Strng):
+    shortVowels = '(' + '|'.join(['\u05B7', '\u05B8', '\u05B4', '\u05BB', '\u05B5', '\u05B6', '\u05B9', '\u05B0']) + ')'
+
+    Strng = re.sub(shortVowels + '(' + 'ו' + ')' + '(?!\u05BC)', r'\1' + 'ב', Strng)
+
+    # Bet with Holam for Vav with beth with holam
+
+    Strng = Strng.replace('בֺ', 'בֹ')
+
+    return Strng
+
+def devanagariuttara(Strng):
+
+    return Strng
+
+def devanagarinepali(Strng):
+
+    return Strng
+
+def devanagaribalbodh(Strng):
+
+    return Strng
+
+def devanagarijain(Strng):
+
+    return Strng
+
+def HiraganaaunotDipthong(Strng):
+
+    return Strng
+
+def IASTISONasalTilde(Strng):
+
+    return Strng
+
+def HeberewQoph(Strng):
+    Strng = Strng.replace('כּ', 'ק').replace('ךּ', 'ק')
+
+    return Strng
+
+def HebewShortO(Strng):
+    Strng = re.sub('(?<!ו)\u05B9', '\u05C7', Strng)
+
+    return Strng
+
+def HebrewKatevMalei(Strng):
+    Strng = Strng.replace('ָ', 'א') # long aa
+    Strng = Strng.replace('ַ', 'א') # short a
+
+    return Strng
+
+def HebrewnonFinalShort(Strng):
+    finals = ['ך', 'ם', 'ן', 'ף', 'ץ', 'ףּ', 'ךּ']
+    finalCons = ['כ', 'מ', 'נ', 'פ', 'צ', 'פּ', 'כּ']
+
+    otherCons = 'ב,ח,ע,צ,ש,ת'.split(',')
+    consonantsAll = '(' + '|'.join(GM.CrunchSymbols(GM.Consonants, 'Hebrew') + finals  + ['׳', 'י', 'ו'] + otherCons) + ')'
+
+    shortVowels = ['\u05B7', '\u05B8', '\u05B4', '\u05BB', '\u05B5', '\u05B6', '\u05B9', '\u05C7']
+    shortVowelsR = '(' + '|'.join(['\u05B7', '\u05B8', '\u05B4', '\u05BB', '\u05B5', '\u05B6', '\u05B9', '\u05C7'] + ['׳']) + ')'
+
+    for s in shortVowels:
+        Strng = re.sub('(' + s + ')' + '(׳?)' + '(?!' + consonantsAll + ')', r'\1\2' + 'ה' + '\u02BE', Strng )
+
+    for f, c in zip(finals, finalCons):
+        Strng = re.sub('(' + f + ')' + shortVowelsR + '(׳?)' + 'ה' + '\u02BE', c + r'\2\3' + 'ה', Strng)
+
+    for f in finals:
+        Strng = Strng.replace(f + '\u05B0', f)
+
+    Strng = Strng.replace('\u05B0' + '׳' + 'ה' + '\u02BE', '\u05B0' + '׳' )
+    Strng = Strng.replace('וֹה' + '\u02BE', 'וֹ' )
+
+    Strng = Strng.replace('\u02BE', '')
+
+    uVowels = ['וֹ', 'וּ']
+
+    for s in uVowels:
+        Strng = re.sub('(' + s + ')' + '(׳?)' + '(?!' + consonantsAll + ')', r'\1\2' + 'א', Strng )
+
+    return Strng
+
+def DevanagariAnusvara(Strng):
+
+    return NasalToAnusvara(Strng, 'Devanagari')
+
+def jainomDevangari(Strng):
+    Strng = Strng.replace('ॐ', 'ꣽ')
+
+    return Strng
+
+def GurmukhiCandrabindu(Strng):
+    Strng = Strng.replace('ਁ', 'ਂ')
+
+    return Strng
+
+def mDotAboveToBelow(Strng):
+    Strng = Strng.replace('ṃ', 'ṁ')
+
+    return Strng
+
+def noLongEO(Strng):
+    Strng = Strng.replace('ē', 'e').replace('ō', 'o')
+
+    return Strng
+
 def TamilStyleUUCore(Strng):
     Strng = re.sub('([ഖഗഘഛഝഠഡഢഥദധഫബഭ])' + '([ുൂ])', r'\1' + '\u200D' + r'\2', Strng)
 
@@ -454,7 +713,6 @@ def TeluguTamilZha(Strng):
 
 def TeluguTamilRra(Strng):
     Strng = Strng.replace('ఱ్ఱ', 'ౘ్ౘ')
-    Strng = Strng.replace('ఱ', 'ౘ')
 
     return Strng
 
@@ -481,6 +739,7 @@ def ThaiNativeConsonants(Strng):
     Strng = Strng.replace('ค', 'ก\u0325')
     Strng = Strng.replace('ช', 'จ\u0325')
 
+    Strng = Strng.replace('ํ', 'ง')
     Strng = Strng.replace('ง', 'งํ')
 
     Strng = Strng.replace('ะงํ\u035C', '\u0E31งํ')
@@ -713,7 +972,7 @@ def SoyomboSpaceTscheg(Strng):
 
     return Strng
 
-def NasaltoAnsvaraIASTISO(Strng):
+def AnusvaratoNasalASTISO(Strng):
     Strng = Strng.replace('ṁ', 'ṃ')
 
     Strng = re.sub('(ṃ)(k|g)', 'ṅ' + r'\2', Strng)
@@ -810,10 +1069,16 @@ def ThaiSajjhayawithA(Strng):
 def LaoSajjhaya(Strng):
     Strng = ThaiSajjhayaOrthography(Strng, Script = "LaoPali")
 
+    Strng = re.sub('([ເໂໄ])(.)(\u0ECE)', r'\2\3\1', Strng)
+
     return Strng
 
 def LaoSajjhayawithA(Strng):
-    Strng = ThaiSajjhayaOrthography(Strng, Script = "LaoPali")
+    Strng = LaoSajjhaya(Strng)
+
+    # The below logic is for Thai yamakkan. Use Thai Yamakkan not to break it
+    Strng = Strng.replace('\u0ECE', '\u0E4E')
+
     Strng = Strng.replace('ັງ', 'ັງ຺')
     Strng = CF.LaoPaliTranscribe(Strng, anusvaraChange = True)
 
@@ -828,6 +1093,9 @@ def LaoSajjhayawithA(Strng):
 
     Strng = Strng.replace('າໍ', 'ຳ')
 
+    # Use Lao Yamakkan again
+    Strng = Strng.replace('\u0E4E', '\u0ECE')
+
     return Strng
 
 def UseAlternateVSU(Strng):
@@ -1188,10 +1456,10 @@ def MalayalamChillu(Strng, reverse=False, preserve=False):
 
     ## may be include ha ?
     CList = [
-            Malayalam.ConsonantMap[10:15]+Malayalam.ConsonantMap[25:27]+Malayalam.ConsonantMap[28:29],
+            Malayalam.ConsonantMap[10:15]+Malayalam.ConsonantMap[24:26]+Malayalam.ConsonantMap[28:29],
             Malayalam.ConsonantMap[15:20]+Malayalam.ConsonantMap[24:27]+Malayalam.ConsonantMap[28:29],
             Malayalam.ConsonantMap[25:27],
-            Malayalam.ConsonantMap[25:28],
+            Malayalam.ConsonantMap[20:21] + Malayalam.ConsonantMap[24:26] + Malayalam.ConsonantMap[27:29],
             Malayalam.SouthConsonantMap[0:1]+Malayalam.ConsonantMap[25:27],
             Malayalam.ConsonantMap[15:20]+Malayalam.ConsonantMap[24:27]+Malayalam.ConsonantMap[28:29]
             ]
@@ -1200,6 +1468,9 @@ def MalayalamChillu(Strng, reverse=False, preserve=False):
         for i in range(len(Chillus)):
             #print '(?<!'+'['+vir+''.join(Chillus)+']'+')'+'('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])'
             Strng = re.sub(ListC + GM.VedicSvaras + '('+ConVir[i]+')'+'(?!['+''.join(CList[i])+'])',r'\1\2' + Chillus[i],Strng)
+            Strng = re.sub(ListC + GM.VedicSvaras + '('+ConVir[i]+')'+'(?=(['+''.join(CList[i])+'])' + vir + r'\4' + ')',r'\1\2' + Chillus[i],Strng)
+
+
     else:
         if preserve:
             for x,y in zip(Chillus, ConVir):
@@ -1287,13 +1558,17 @@ def GurmukhiTippiGemination(Strng):
 
     return Strng
 
+def khandatabatova(Strng):
+    Strng = Strng.replace('ৎব', 'ত্ৱ')
+
+    return Strng
+
 def KhandaTa(Strng,Target, reverse=False): #Check for Bhakt - Khanda Ta not formed
 
     ta = GM.CrunchSymbols(GM.Consonants, Target)[15]
     khandata = '\u09CE'
     vir = GM.CrunchSymbols(GM.VowelSigns,Target)[0]
-    ListC = '|'.join([GM.CrunchList('ConsonantMap', Target)[x] for x in [15,16,19,22,27,24,25,26]] + ['ৰ'])
-    #print ListC
+    ListC = '|'.join([GM.CrunchList('ConsonantMap', Target)[x] for x in [15,16,19,27,24,25,26,28]] + ['ৰ', 'য়'])
     if not reverse:
         Strng = re.sub('(?<!' + vir + ')' + '('+ta+')'+'('+vir+')'+'(?!'+ListC+')',khandata, Strng)
     else:
@@ -1485,10 +1760,20 @@ def VaToBa(Strng,Target):
     va = GM.CrunchSymbols(GM.Consonants, Target)[28]
     ba = GM.CrunchSymbols(GM.Consonants, Target)[22]
 
+    if Target == 'Bengali':
+        #Strng = Strng.replace('ৎৱ', 'ত্ব')
+        pass
+
     Strng = Strng.replace(va,ba)
 
     return Strng
 
+def tbadiff(Strng,Target):
+
+    Strng = Strng.replace('ৎব', 'ত্ব')
+
+    return Strng
+
 def RetainDandasIndic(Strng, Target, reverse=False):
     Dandas = GM.CrunchList('SignMap', Target)[1:3]
 
@@ -1783,11 +2068,50 @@ def ThamTallAOthers(Strng):
     return Strng
 
 def LaoPhonetic(Strng):
-    Strng = Strng.replace('ຄ', 'ກ')
-    Strng = Strng.replace('ຊ', 'ຈ')
+    Strng = re.sub('(\u0EBA)([ໂເໄ]?)([ຍຣລວຫ])', '\u035C'+ r'\2\3', Strng)
+    Strng = re.sub('([ຍຣລວຫ])' + '\u035C' + '([ໂເໄ]?)' + r'\1', r'\1' + '\u0EBA' + r'\2\1', Strng)
+
+    Strng = Strng.replace('ຫ\u0EBA', 'ຫ\u035C')
+
+    Strng = re.sub('([ຍຣລວຫ])' + '\u035C' + r'\1', r'\1' + '\u0EBA' + r'\1', Strng)
+
+    Strng = LaoTranscription(Strng)
+
+    Strng = Strng.replace('\u0EB0\u035C', '\u035C')
+
+    Strng = Strng.replace('ງ', 'ງໍ')
+
+    #Strng = Strng.replace('ຄ', 'ກ')
+    #Strng = Strng.replace('ຊ', 'ຈ')
     Strng = Strng.replace('ທ', 'ດ')
     Strng = Strng.replace('ພ', 'ບ')
-    Strng = Strng.replace('\u0ECD', 'ງໍ')
+
+    return Strng
+
+
+def RephaDoubleMalayalam(Strng):
+    repha = '[ർൎ]'
+
+    Target = 'Malayalam'
+
+    vir = GM.CrunchSymbols(GM.VowelSigns,Target)[0]
+    ConUnAsp = [GM.CrunchList('ConsonantMap', Target)[x] for x in [0,2,5,7,10,12,15,17,20,22,4,9,14,19,24,25,28,29,31]]
+    ConUnAsp = ConUnAsp + ['ള']
+    ConAsp   = [GM.CrunchList('ConsonantMap', Target)[x] for x in [1,3,6,8,11,13,16,18,21]]
+
+    Strng = re.sub('(' + repha + ')' + '('+'|'.join(ConUnAsp)+')', r'\1\2' + vir + r'\2', Strng)
+
+    for i in range(len(ConAsp)):
+        Strng = re.sub('(' + repha + ')' + '(' + ConAsp [i] +')', r'\1' +  ConUnAsp[i] + vir + r'\2', Strng)
+
+    # Dot reph with ya
+
+    # Strng = Strng.replace('ൎയ', 'ൎയ്യ')
+
+    return Strng
+
+def DograShaKha(Strng):
+    Strng = Strng.replace('𑠨', '𑠋')
 
     return Strng
 
@@ -1800,6 +2124,8 @@ def ThamShiftMaiKangLai(Strng):
     Strng = re.sub('(\u1A58)' + ListV, r'\2\1', Strng)
     Strng = re.sub('(\u1A58)' + ListV, r'\2\1', Strng)
 
+    #Strng = Strng.replace('\u1A63\u1A58', '\u1A58\u1A63')
+
     return Strng
 
 def FixTallA(Strng, TallACons):
@@ -1871,6 +2197,8 @@ def ThaiSajjhayaOrthography(Strng, Script = "Thai"):
         Strng = Strng.replace("์", "໌")
         Strng = re.sub('(\u0EB1)(.)(\u0E4E)', r'\2\3', Strng)
 
+        Strng = Strng.replace('\u0E4E', '\u0ECE')
+
     #Strng = re.sub('([ยรลวศษสหฬ])(์)', r'\1' + '๎', Strng)
 
     return Strng
