@@ -13,6 +13,90 @@ def ShowChillus(Strng):
 
     return PostProcess.MalayalamChillu(Strng, True, True)
 
+def eiaudipthongs(Strng):
+
+    return Strng
+
+def wasvnukta(Strng):
+
+    return Strng
+
+def default(Strng):
+
+    return Strng
+
+def JapanesePreProcess(src, txt, preoptions):
+    import pykakasi
+    from . import Convert
+
+    if src == 'Hiragana' or src == 'Katakana':
+        # preserve roman words
+        kks = pykakasi.kakasi()
+
+        txt = Convert.convertScript(txt.lower(), "ISO", "Devanagari")
+        #print(txt)
+
+        cv = kks.convert(txt)
+        txt = ''
+
+        for item in cv:
+            txt = txt + ' ' + item['hepburn']
+
+        if 'eiaudipthongs' in preoptions:
+            txt = txt.replace('ou', 'o\u02BDu').replace('ei', 'e\u02BDi')
+
+        txt = re.sub('(r)([aiueo])(\u309A\u309A)', 'l' + r'\2\2', txt)
+        txt = re.sub('(r)([aāiīuūeēoō])(\u309A)', 'l' + r'\2', txt)
+
+        txt = re.sub('(k)([aiueo])(\u309A\u309A)', 'ṅ' + r'\2\2', txt)
+        txt = re.sub('(k)([aāiīuūeēoō])(\u309A)', 'ṅ' + r'\2', txt)
+
+        txt = txt.replace('aa', 'ā').replace('ii', 'ī').replace('ee', 'ē').replace('oo', 'ō').replace('uu','ū')
+        txt = txt.replace('a-', 'ā').replace('i-', 'ī').replace('e-', 'ē').replace('o-', 'ō').replace('u-','ū')
+        txt = txt.replace("n'", 'n_').replace('ch', 'c').replace('sh', 'ṣ').replace('sṣ', 'ṣṣ').replace('ai', 'a_i').replace('au', 'a_u')
+        txt = txt.replace('w', 'v')
+        txt = txt.replace('ou', 'ō').replace('ei', 'ē')
+        txt = txt.replace('、', ',').replace('。', '.')
+
+        # Nasalization of n
+
+        txt= txt.replace('ng', 'ṅg').replace('nk', 'ṅk').replace('nk', 'ṅk').replace('np', 'mp').replace('nb', 'mb').replace('nm', 'mm')
+
+        if 'wasvnukta' in preoptions:
+            txt = txt.replace('v', 'v̈')
+
+        #print(txt)
+
+        txt = txt.replace('、', ',').replace('。', '.')
+
+    return txt
+
+def holamlong(Strng):
+    Strng = Strng.replace('ֹּ','ֹּ')
+    Strng = re.sub('(?<!ו)ֹ', 'וֹ', Strng)
+
+    return Strng
+
+def novowelshebrew(Strng):
+    finals = ['ך', 'ם', 'ן', 'ף', 'ץ', 'ףּ', 'ךּ']
+    otherCons = 'ב,ח,ע,צ,ש,ת'.split(',')
+    consonantsAll = '(' + '|'.join(GM.CrunchSymbols(GM.Consonants, 'Hebrew') + finals  + otherCons + ['׳', 'י', 'ו' ,'א']) + ')'
+    vowelsignsADShinG = '(' + '|'.join(GM.CrunchSymbols(GM.VowelSigns, 'Hebrew') + ['ַ', 'ּ', 'ׁ', '׳']) + ')'
+
+    Strng = re.sub(consonantsAll + "(?!" + vowelsignsADShinG + ')', r'\1' + 'ַ' + r'\2', Strng)
+
+    return Strng
+
+def shvanakhall(Strng):
+    Strng = Strng + ' \u0BDE'
+
+    return Strng
+
+def longEOISO(Strng):
+    Strng = Strng.replace('e', 'ē').replace('o', 'ō')
+
+    return Strng
+
 def SanskritLexicaizeHK(Strng):
 
     return Strng
@@ -32,6 +116,17 @@ def ThaiPhonetic(Strng):
 
     return Strng
 
+def LaoPhonetic(Strng):
+    Strng = Strng.replace('ດ', 'ທ')
+    Strng = Strng.replace('ບ', 'ພ')
+    Strng = Strng.replace('ງໍ', 'ງ')
+
+    Strng = Strng.replace('\u035C', '')
+
+    Strng += "\u02BB\u02BB"
+
+    return Strng
+
 def SaurastraHaaruColonTamil(Strng):
     Strng = Strng.replace('ன', 'ந')
 
@@ -373,6 +468,11 @@ def PreProcess(Strng,Source,Target):
     if Source in GM.RomanDiacritic:
         Strng = Strng.lower()
 
+    pipeScripts = ["HK", "IASTPali", "IAST", "ISO"]
+
+    if Source in pipeScripts:
+        Strng = Strng.replace("|", ".").replace("||", "..")
+
     if Source == 'Itrans':
         sOm = 'OM'
         tOm = 'oM'
@@ -393,21 +493,51 @@ def PreProcess(Strng,Source,Target):
         Strng = re.sub('^' + sOm + '$', tOm, Strng)
 
 
-        AltForm = ['O', 'aa','ii','uu','RRi','RRI','LLi','LLI','N^','JN','chh','shh','x','GY','.n','.m','.h', 'AUM', 'E']
-        NormForm = ['^o', 'A','I','U','R^i','R^I','L^i','L^I','~N','~n','Ch','Sh','kSh','j~n','M','M','','oM', '^e']
+        AltForm = ['O', 'aa','ii','uu','RRi','RRI','LLi','LLI','N^','JN','chh','shh','x','GY','.n','.m','.h', 'AUM', 'E', 'J']
+        NormForm = ['^o', 'A','I','U','R^i','R^I','L^i','L^I','~N','~n','Ch','Sh','kSh','j~n','M','M','','oM', '^e', 'z']
 
         for x,y in zip(AltForm,NormForm):
             Strng = Strng.replace(x,y)
 
         Strng = Strng.replace('OM', 'oM')
 
+    if Source == 'BarahaNorth' or Source == 'BarahaSouth':
+        Strng = Strng.replace('A', 'aa')
+        Strng = Strng.replace('I', 'ee')
+        Strng = Strng.replace('U', 'oo')
+
+        Strng = Strng.replace('ou', 'au')
+        Strng = Strng.replace('K', 'kh')
+        Strng = Strng.replace('G','gh')
+        Strng = Strng.replace('ch', 'c')
+        Strng = Strng.replace('Ch', 'C')
+        Strng = Strng.replace('J','jh')
+        Strng = Strng.replace('w', 'v')
+        Strng = Strng.replace('sh', 'S')
+        Strng = Strng.replace('~h', '_h')
+        Strng = Strng.replace('^', '()')
+        Strng = Strng.replace('^^', '{}')
+
+        Strng = Strng.replace('tx', 'rx')
+        Strng = Strng.replace('zh', 'Lx')
+
+        Strng = Strng.replace('q', '\_')
+        Strng = Strng.replace('#', "\\'")
+        Strng = Strng.replace('$', '\\"')
+
+
     if Source == 'IAST':
         Strng = Strng.replace("aï", "a_i")
         Strng = Strng.replace("aü", "a_u")
+        Strng = Strng.replace('\u0303', 'ṃ')
 
     if Source == "ISO":
         Strng = Strng.replace('a:i', 'a_i')
         Strng = Strng.replace('a:u', 'a_u')
+        Strng = Strng.replace('\u0303', 'ṁ')
+
+    if Source == "Titus":
+        Strng = Strng
 
     if Source == "ISO" or Source == "IAST" or Source == "Titus" or "RussianCyrillic":
         Strng = CF.VedicSvarasNonDiacritic(Strng)
@@ -647,6 +777,7 @@ def TamilTranscribe(Strng):
         pass
         #Strng = re.sub('('+Vowels+Consonants+')'+ConUnVoiced[i]+'('+Vowels+Consonants+')',r'\1'+ConVoicedS[i]+r'\2',Strng)
         Strng = re.sub('('+Vowels+'|'+Consonants+'|'+Aytham+')'+ConUnVoiced[i]+'(?!'+vir+')',r'\1'+ConVoicedS[i],Strng)
+        Strng = re.sub('([³])'+ConUnVoiced[i]+'(?!'+vir+')',r'\1'+ConVoicedS[i],Strng)
 
         # Nasals + Unvoiced -> voiced
         # Rule applied even if spaced -> ulakaJ cuRRu -> ulakaJ juRRu; cenJu -> senju
