@@ -8,6 +8,7 @@ import itertools
 from collections import Counter
 import unicodedata
 import io
+import collections
 
 def removeA(a):
     if a.count('a') == 1:
@@ -183,12 +184,18 @@ def detect_preoptions(text, inputScript):
 def convert(src, tgt, txt, nativize, preoptions, postoptions):
     tgtOld = ""
 
+    if src == "Itrans" and '##' in txt:
+        textNew = [[i,word] for i, word in enumerate(txt.split("##")) if (i%2 == 0)]
+        textRest = [(i,word) for i, word in enumerate(txt.split("##")) if (i%2 == 1)]
+
+        txt = json.dumps(textNew).replace("\\n", "\n")
+
     if tgt == "" or tgt == "Ignore":
         return txt
     if preoptions == [] and postoptions == [] and nativize == False and src == tgt:
         return txt
 
-    if src == tgt:
+    if src == tgt and (src != 'Hiragana' and src != 'Katakana'):
         tgtOld = tgt
         tgt = "Devanagari"
 
@@ -220,10 +227,32 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
     for options in preoptions:
       txt = getattr(PreProcess, options)(txt)
 
+    srcOld = ""
+
+    if src == 'Hiragana' or src == 'Katakana':
+        txt = PreProcess.JapanesePreProcess(src, txt, preoptions)
+        srcOld = 'Japanese'
+        src = 'ISO'
+
+    if tgt == 'Hiragana' or tgt == 'Katakana':
+        txt = PostProcess.JapanesePostProcess(src, tgt, txt, nativize, postoptions)
+
     if src == "Oriya" and tgt == "IPA":
         txt = ConvertFix.OriyaIPAFixPre(txt)
 
-    transliteration = Convert.convertScript(txt, src, tgt)
+    if src == "Itrans" and '##' in txt:
+        #print('I am here tyring to do things')
+        transliteration = ''
+        for i, word in enumerate(txt.split("##")):
+            if (i%2 == 0):
+                transliteration += Convert.convertScript(word, src, tgt)
+            else:
+                transliteration += word
+    else:
+        transliteration = Convert.convertScript(txt, src, tgt)
+
+    if srcOld == 'Japanese' and tgt != 'Devanagari' and 'siddhammukta' not in postoptions:
+        transliteration = Convert.convertScript(transliteration, "Devanagari", "ISO")
 
     if src == tgtOld:
         tgt = tgtOld
@@ -251,6 +280,11 @@ def convert(src, tgt, txt, nativize, preoptions, postoptions):
     if src == "Oriya" and tgt == "IPA":
         transliteration = ConvertFix.OriyaIPAFix(transliteration)
 
+    if src == "Itrans" and '##' in txt:
+        textConv = {**dict(list(json.loads(transliteration.replace("\n", "\\n")))), ** dict(textRest)}
+        textConv = collections.OrderedDict(sorted(textConv.items()))
+        transliteration = "".join(textConv.values())
+
     return transliteration
 
 def process(src, tgt, txt, nativize = True, post_options = [], pre_options = []):
